unit Group;

interface
uses Windows, GIntData, CommDlg;

const PleiadesData = 'pleiades.dat'; {      }
      PleiadesSection = 'Pleiades area'; {     StarCalc,
                                                }
      PleiadesValueName = 'DataFileName'; {    
                                             -     . }
      J2000 = 0.0; { ,        }

{       }
type PStarData = ^TStarData;
     TStarData = packed record
        Space1 :char;
        RaHour :array[1..2] of char;
        Space2 :char;
        RaMin  :array[1..2] of char;
        Space3 :char;
        RaSec  :array[1..6] of char;
        Space4 :array[1..2] of char;
        DecSign:char;
        DecDeg :array[1..2] of char;
        Space5 :char;
        DecMin :array[1..2] of char;
        Space6 :char;
        DecSec :array[1..5] of char;
        Space7 :array[1..3] of char;
        RMag   :array[1..4] of char;
        RMag_c :char;
        Space8 :char;
        BMag   :array[1..4] of char;
        BMag_c :char;

        CRLF :array[1..2] of char
     end;

     {    ,        }
     TStarImage = record
        Ra, Dec :Double;
        RMag, BMag :Integer;
        RMag_c, BMag_c :char;

        Visible :Boolean;
        Rect :TRect;
        x,y :Integer;
     end;

     {       }
     TStarImages = array[1..15000] of TStarImage;

type TPleiadesArea = class(TObjectGroup)
        StarImages :^TStarImages; {     
                                   (    ) }
        procedure Init; override;
        function InitResources(Parent :HWnd) :Boolean; override;
        procedure FreeResources(Parent :HWnd); override;

        procedure CalculateObjects(SetObjectNum :TSetObjectNum); override;
        procedure DrawObjects(GetObjectNum :TGetObjectNum); override;

        function GetObjectRect(ObjectNum :Integer; var Rect :TRect) :Boolean; override;
        function GetObjectRaDec(ObjectNum :Integer; var Ra,Dec :Double) :Boolean; override;

        function GetObjectData(ObjectNum :Integer; var Buffer; BufferSize :Integer) :Integer; override;
        procedure GetObjectStrings(ObjectNum :Integer; const info :TZStrings); override;
        function GetObjectName(ObjectNum :Integer) :string; override;

        procedure AdvancedInfo(Parent :HWnd; ObjectNum, FunctionNum :Integer); override;
     end;

var wGroup :TPleiadesArea;

implementation

{    , ,  ,    
  SysUtils,            
  .     dll,   . }
function StrLen(Str :PChar) :Integer;
begin
   Result := 0;
   if Str<>nil then
      while Str[Result]<>#0 do Inc(Result)
end;

function Trim(const S :string) :string;
var i,j :Integer;
begin
   i:=1; while (i<=Length(S)) and (S[i]=' ') do Inc(i);
   j:=Length(S); while (j>=i) and (S[j]=' ') do Dec(j);
   Trim := Copy(S, i, j-i+1);
end;

function IntToStr(Value :Integer) :string;
begin
   Str(Value:0, Result);
end;

function FloatToStr(Value :Double; Decimal :Integer) :string;
begin
   Str(Value:0:Decimal, Result);
end;

{    }

procedure TPleiadesArea.Init;
begin
   ID := 'PLDES';
   Description := ' ';
   Version := '1.0';
   EpochType := epFixed;
   Epoch := J2000;

   AdvancedInfoStrings.Add('/');
end;

function TPleiadesArea.InitResources(Parent :HWnd) :Boolean;
var PlFilePath, FullPlFName :string;
    OpenFileName :TOpenFileName;
    Filter :String;
    Code, i,j :Integer;
    tmpValue :Double;

    MapFile :hFile;
    MapHandle :THandle;
    FileBlock :PChar;

    Error :Boolean;
begin
   Result := false;

   {      StarCalc }
   SetLength(PlFilePath, 255);
   SetLength(PlFilePath, GetModuleFileName(HInstance, PChar(PlFilePath), 255));
   while (Length(PlFilePath)>0) and (PlFilePath[Length(PlFilePath)]<>'\') do
      SetLength(PlFilePath, Length(PlFilePath)-1);

   SetLength(FullPlFName, 255);
   SetLength(FullPlFName, RestoreIniString(PleiadesSection, PleiadesValueName,
      PChar(PlFilePath+PleiadesData), PChar(FullPlFName)^, 255));

   {     ,      . }
   if GetFileAttributes(PChar(FullPlFName))<0 then begin
      FullPlFName := PleiadesData+#0;
      SetLength(FullPlFName, 255);

      Filter := '   '#0+PleiadesData+#0;

      FillChar(OpenFileName, SizeOf(OpenFileName), #0);
      OpenFileName.lStructSize := SizeOf(OpenFileName);
      OpenFileName.hwndOwner := Parent;
      OpenFileName.hInstance := HInstance;
      OpenFileName.lpstrFilter := PChar(Filter);
      OpenFileName.lpstrFile := PChar(FullPlFName);
      OpenFileName.nMaxFile := 255;
      OpenFileName.nMaxFileTitle := 255;
      OpenFileName.lpstrInitialDir := PChar(PlFilePath);
      OpenFileName.lpstrTitle := 'Select file:';
      OpenFileName.Flags := OFN_HIDEREADONLY+OFN_LONGNAMES+OFN_NOCHANGEDIR;

      if not GetOpenFileName(OpenFileName) then Exit;

      SetLength(FullPlFName, StrLen(PChar(FullPlFName)));
      SaveIniString(PleiadesSection, PleiadesValueName, PChar(FullPlFName))
   end;

   {   .          
     ,         Win32 API,
            . }

   MapFile := CreateFile(PChar(FullPlFName), GENERIC_READ, FILE_SHARE_READ, nil,
                         OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
   MapHandle := CreateFileMapping(MapFile, nil, PAGE_READONLY, 0, 0, nil);
   FileBlock := MapViewOfFile(MapHandle, FILE_MAP_READ, 0, 0, 0);

   Result := (MapFile<>0) and (MapHandle<>0) and (FileBlock<>nil);

   if Result then begin
      RecordCount := GetFileSize(MapFile, nil) div SizeOf(TStarData);
      GetMem(StarImages, SizeOf(TStarImage)*RecordCount);

      j := 0; Error := false;
      for i:=1 to RecordCount do begin
         with PStarData(FileBlock+j)^ do begin
            if CRLF<>#13#10 then begin Error := true; break end;

            Val(Trim(RaHour), StarImages[i].Ra, Code);

            Val(Trim(RaMin), tmpValue, Code);
            StarImages[i].Ra := StarImages[i].Ra + tmpValue/60;

            Val(Trim(RaSec), tmpValue, Code);
            StarImages[i].Ra := StarImages[i].Ra + tmpValue/3600;

            StarImages[i].Ra := StarImages[i].Ra*15;

            Val(Trim(DecDeg), StarImages[i].Dec, Code);

            Val(Trim(DecMin), tmpValue, Code);
            StarImages[i].Dec := StarImages[i].Dec + tmpValue/60;

            Val(Trim(DecSec), tmpValue, Code);
            StarImages[i].Dec := StarImages[i].Dec + tmpValue/3600;

            if DecSign='-' then StarImages[i].Dec := -StarImages[i].Dec;

            Val(Trim(RMag), tmpValue, Code);
            StarImages[i].RMag := Round(tmpValue*100);
            StarImages[i].RMag_c := RMag_c;

            Val(Trim(BMag), tmpValue, Code);
            StarImages[i].BMag := Round(tmpValue*100);
            StarImages[i].BMag_c := BMag_c
         end;
         Inc(j, SizeOf(TStarData))
      end;

      Result := not Error;

      if not Result then FreeMem(StarImages, SizeOf(TStarImage)*RecordCount);
   end;

   UnmapViewOfFile(FileBlock);
   CloseHandle(MapHandle);
   CloseHandle(MapFile);
end;

procedure TPleiadesArea.FreeResources(Parent :HWnd);
begin
   FreeMem(StarImages, SizeOf(TStarImage)*RecordCount);
   RecordCount := 0;
end;

procedure TPleiadesArea.CalculateObjects(SetObjectNum :TSetObjectNum);
var i, Width,Height :Integer;
begin
   for i:=1 to RecordCount do
      with StarImages[i] do begin
         Visible := DrawStars and ((RMag<=MaxStarMag) or (MaxStarMag=2000));

         if Visible then begin
            if ViewMode=QUICK then
               Visible := QuickRaDecToXY(Round(Ra*60), Round(Dec*60), x,y)
            else
               if IsEqPointVisible(Ra,Dec) then
                  Visible := RaDecToXY(Ra,Dec, x,y)
               else
                  Visible := false;
         end;

         if Visible then begin
            GetSymbolSize(MagToSymbolNum(RMag), Width,Height);
            System.Dec(x, Width div 2);
            System.Dec(y, Height div 2);
            SetRect(Rect, x,y, x+Width,y+Height);

            SetObjectNum(i);
         end;
      end;
end;

procedure TPleiadesArea.DrawObjects(GetObjectNum :TGetObjectNum);
var ObjNum :Integer;
begin
   ObjNum := GetObjectNum;
   while ObjNum<>OBJECTNUM_ERROR do begin

      with StarImages[ObjNum] do
         if IsRectVisible(Rect) then
            DrawSymbol(MagToSymbolNum(RMag), x,y, StdColors[clStarImages]);

      ObjNum := GetObjectNum;
   end;
end;

function TPleiadesArea.GetObjectRect(ObjectNum :Integer; var Rect :TRect) :Boolean;
begin
   Result := ObjectNumValid(ObjectNum) and DrawStars and StarImages[ObjectNum].Visible;
   if Result then Rect := StarImages[ObjectNum].Rect
end;

function TPleiadesArea.GetObjectRaDec(ObjectNum :Integer; var Ra,Dec :Double) :Boolean;
begin
   Result := ObjectNumValid(ObjectNum);
   if Result then begin
      Ra  := StarImages[ObjectNum].Ra;
      Dec := StarImages[ObjectNum].Dec;
   end;
end;

function TPleiadesArea.GetObjectData(ObjectNum :Integer; var Buffer; BufferSize :Integer) :Integer;
var StarInfo :packed record
       Ra,Dec :Double;
       RMag,BMag :Double;
       RMag_c, BMag_c :char;
    end absolute Buffer;
begin
   if ObjectNumValid(ObjectNum) and (BufferSize<SizeOf(StarInfo)) then
      with StarImages[ObjectNum] do begin
         StarInfo.Ra     := Ra;
         StarInfo.Dec    := Dec;
         StarInfo.RMag   := RMag;
         StarInfo.RMag_c := RMag_c;
         StarInfo.BMag   := BMag;
         StarInfo.BMag_c := BMag_c;

         Result := SizeOf(StarInfo);
      end
   else
      Result := 0;
end;

procedure TPleiadesArea.GetObjectStrings(ObjectNum :Integer; const info :TZStrings);
var tmpIntValue :Integer;
begin
   if not ObjectNumValid(ObjectNum) then Exit;

   info.Add(tsNorm+'  ');

   with StarImages[ObjectNum] do begin
      info.Add(tsNorm+'.  (J2000):');

      tmpIntValue := Round(Ra/15*3600*1000);
      info.AddSubstr(tsGreek+'a'+tsNorm+': '+
         IntToStr(tmpIntValue div 3600000) + '' +
         IntToStr(tmpIntValue mod 3600000 div 60000) + '' +
         FloatToStr((tmpIntValue mod 60000)/1000, 3) + '   ');

      tmpIntValue := Round(Abs(Dec)*3600*100);
      info.AddSubstr(tsGreek+'d'+tsNorm+': ');
      if Dec<0 then info.AddSubstr('-') else info.AddSubstr('+');
      info.Add(IntToStr(tmpIntValue div 360000) + '' +
               IntToStr(tmpIntValue mod 360000 div 6000) + '''' +
               FloatToStr((tmpIntValue mod 6000)/100, 2) + '"');

      info.Add(tsNorm+'R: '+FloatToStr(RMag/100,1)+RMag_c+tsIndexOn+'m'+tsIndexOff);
      info.Add(tsNorm+'B: '+FloatToStr(BMag/100,1)+BMag_c+tsIndexOn+'m'+tsIndexOff);
   end;
end;

function TPleiadesArea.GetObjectName(ObjectNum :Integer) :string;
begin
   if ObjectNumValid(ObjectNum) then
      with StarImages[ObjectNum] do
         Result := tsNorm+'   '+
            '(R:' + FloatToStr(RMag/100,1)+RMag_c+tsIndexOn+'m'+tsIndexOff + ';' +
            ' B:' + FloatToStr(BMag/100,1)+BMag_c+tsIndexOn+'m'+tsIndexOff + ')'
   else
      Result := '';
end;

var RiseSettStar :Integer;
procedure GetStarRaDec(J :Double; var tRa,tDec :Double); stdcall;
begin
   with wGroup.StarImages[RiseSettStar] do begin
      tRa := Ra; tDec := Dec;
   end;
end;

procedure TPleiadesArea.AdvancedInfo(Parent :HWnd; ObjectNum, FunctionNum :Integer);
begin
   if ObjectNumValid(ObjectNum) then begin
      RiseSettStar := ObjectNum;
      ShowRiseSettDialog(-34/60, Epoch, @GetStarRaDec);
   end;
end;

initialization
   wGroup := TPleiadesArea.Create;
finalization
   wGroup.Free;
end.
