unit Group;

interface
uses Windows, GIntData;

const {  . }
      NorthPole = ' ';
      SouthPole = ' ';

      {  ,    }
      ImageSize = 4;

type  {    }
      TPoleImageData = record
         Visible :Boolean; {      }
         x,y :Integer; {     }
         R :TRect; { ,      }
      end;

type TPoleType = (NORTH_POLE, SOUTH_POLE); {   }

type TPole = class(TObjectGroup)
        {       }
        ImageData :array[TPoleType] of TPoleImageData;

        procedure Init; override;
        procedure CalculateObjects(SetObjectNum :TSetObjectNum); override;
        procedure DrawObjects(GetObjectNum :TGetObjectNum); override;
        function GetObjectRect(ObjectNum :Integer; var Rect :TRect) :Boolean; override;
        procedure GetObjectStrings(ObjectNum :Integer; const info :TZStrings); override;
        function GetObjectName(ObjectNum :Integer) :string; override;
     end;

var wGroup :TPole;

implementation

{    }
procedure TPole.Init;
begin
   ID := 'POLE';
   Description := '  ';
   Version := '1.0';
   EpochType := epTrueCurrent;
   RecordCount := 2;
end;

procedure TPole.CalculateObjects(SetObjectNum :TSetObjectNum);

{     }
function CalcPoleImage( p :TPoleType) :Boolean;
var Dec :Double;
begin
   if p=NORTH_POLE then Dec := +90.0 else Dec := -90.0;

   with ImageData[p] do begin
      {     (" "/" ") 
               QuickRaDecToXY 
        RaDecToXY }
      if ViewMode=QUICK then
         Visible := QuickRaDecToXY(0, Round(Dec*60.0), x,y)
      else
         Visible := RaDecToXY(0, Dec, x,y);

      if Visible then SetRect(R, x-ImageSize,y-ImageSize,x+ImageSize+1,y+ImageSize+1);

      Result := Visible;
   end;
end;

begin
   if CalcPoleImage(NORTH_POLE) then SetObjectNum(1);
   if CalcPoleImage(SOUTH_POLE) then SetObjectNum(2);
end;

procedure TPole.DrawObjects(GetObjectNum :TGetObjectNum);
var ObjNum :Integer;
    oldColor :TColor;
begin
   {  ,  ,    }
   oldColor := SetDrawColor(atPen, RGB(255,255,255));

   ObjNum := GetObjectNum;
   while ObjNum<>OBJECTNUM_ERROR do begin

      with ImageData[TPoleType(ObjNum-1)] do
         {            ,
             ,  . }
         if Visible and IsRectVisible(R) then begin

            MoveToEx(DC, x-ImageSize, y-ImageSize, nil);
            LineTo(DC, x+ImageSize+1, y+ImageSize+1);

            MoveToEx(DC, x-ImageSize, y+ImageSize, nil);
            LineTo(DC, x+ImageSize+1, y-ImageSize-1);

         end;

      ObjNum := GetObjectNum;
   end;

   {       pen' }
   SetDrawColor(atPen, oldColor);
end;

function TPole.GetObjectRect(ObjectNum :Integer; var Rect :TRect) :Boolean;
begin
   Result := false;
   if ObjectNumValid(ObjectNum) then
      with ImageData[TPoleType(ObjectNum-1)] do
         if Visible then begin
            Rect := R;
            Result := true;
         end;
end;

{  ( )    }
procedure TPole.GetObjectStrings(ObjectNum :Integer; const info :TZStrings);
begin
   if ObjectNum=1 then info.Add(NorthPole);
   if ObjectNum=2 then info.Add(SouthPole);
end;

{            }
function TPole.GetObjectName(ObjectNum :Integer) :string;
begin
   Result := '';
   if ObjectNum=1 then Result := NorthPole;
   if ObjectNum=2 then Result := SouthPole;
end;

initialization
   wGroup := TPole.Create;
finalization
   wGroup.Free;
end.
