unit GExports;

interface
uses Windows, GIntData, Group;

const GI_DESCRIPTION      = 0;
      GI_RECORDCOUNT      = 1;
      GI_SEARCHDESCSTRING = 2;
      GI_ADVACTIONSTRING  = 3;
      GI_ADVINFOSTRINGS   = 4;

      OI_RECT    = 0;
      OI_RADEC   = 1;
      OI_XY      = 2;
      OI_DATA    = 3;
      OI_STRINGS = 4;
      OI_NAME    = 5;

type TSearchData = packed record
         GroupID :TGroupID;
         Ra,Dec :Double;
     end;

function GetGroupInfo(InfoType :Integer; var Buffer; BufferSize :Integer) :Integer; stdcall;

function GetObjectInfo(ObjNum :Integer; InfoType :Integer; var Buffer; BufferSize :Integer) :Integer; stdcall;

function InitResources(Parent :HWND) :Boolean; stdcall;

procedure FreeResources(Parent :HWND); stdcall;

procedure RefreshParam(Param :PCommonParameters); stdcall;

procedure CalculateObjects(ViewMode :TViewMode; ImageRect :TRect;
           MinRa, MaxRa, MinDec, MaxDec :Double; SetObjectNum :TSetObjectNum); stdcall;

procedure DrawObjects(DC :HDC; const CRect :TRect; GetObjectNum :TGetObjectNum); stdcall;

function SelectObjectForSearch(Parent :HWND; var Data :TSearchData) :Integer; stdcall;

function EditOptions(Parent :HWND) :Boolean; stdcall;

procedure AdvancedInfo(Parent :HWND; ObjNum, FuncNum :Integer); stdcall;

function AdvancedAction(Parent :HWND) :Boolean; stdcall;

exports
    GetGroupInfo          index  1 name 'GetGroupInfo',
    GetObjectInfo         index  2 name 'GetObjectInfo',
    InitResources         index  3 name 'InitResources',
    FreeResources         index  4 name 'FreeResources',
    RefreshParam          index  5 name 'RefreshParam',
    CalculateObjects      index  6 name 'CalculateObjects',
    DrawObjects           index  7 name 'DrawObjects',
    SelectObjectForSearch index  8 name 'SelectObjectForSearch',
    EditOptions           index  9 name 'EditOptions',
    AdvancedInfo          index 10 name 'AdvancedInfo',
    AdvancedAction        index 11 name 'AdvancedAction';

implementation

function SetStrField(var Buffer; BufferSize :Integer; const S :string) :Integer;
var cnt :Integer;
begin
   FillChar(Buffer, BufferSize, #0);
   cnt := Length(S);
   if cnt>0 then begin
      if cnt>=BufferSize then cnt := BufferSize-1;
      Move(PChar(S)^, Buffer, cnt);
      Result := cnt+1;
   end else
      Result := 0;
end;

function GetGroupInfo(InfoType :Integer; var Buffer; BufferSize :Integer) :Integer;
var GroupInfo :TGroupInfo absolute Buffer;
begin
   Result := 0;

   case InfoType of
   GI_DESCRIPTION :
      with GroupInfo do begin
         SetStrField(GroupID, SizeOf(TGroupID), wGroup.ID);
         SetStrField(Description, SizeOf(Description), wGroup.Description);
         SetStrField(Version, SizeOf(Version), wGroup.Version);

         RecordCount := wGroup.GetRecordCount;
         EpochType   := wGroup.EpochType;
         Epoch       := wGroup.Epoch;

         sccFlags := 0;
         if wGroup.InternalOptions then Inc(sccFlags, ciInternalOptions);
         if wGroup.AdvancedInfoStrings.Count>0 then Inc(sccFlags, ciAdvancedInfo);
         if wGroup.AdvancedActionString<>'' then Inc(sccFlags, ciAdvancedAction);
         if wGroup.ObjectSelect then Inc(sccFlags, ciObjSelect);
         if wGroup.SearchDescrString<>'' then Inc(sccFlags, ciObjSearch);
         if wGroup.QuickView then Inc(sccFlags, ciQuickView);
         if wGroup.DetailedView then Inc(sccFlags, ciDetailedView);

         Init := wGroup.ResourcesInited;
      end;
   GI_RECORDCOUNT :
      Result := wGroup.GetRecordCount;
   GI_SEARCHDESCSTRING :
      Result := SetStrField(Buffer, BufferSize, wGroup.SearchDescrString);
   GI_ADVACTIONSTRING :
      Result := SetStrField(Buffer, BufferSize, wGroup.AdvancedActionString);
   GI_ADVINFOSTRINGS :begin
      SetStrField(Buffer, BufferSize, wGroup.AdvancedInfoStrings.Str);
      Result := wGroup.AdvancedInfoStrings.Count;
   end
   end;
end;

function GetObjectInfo(ObjNum :Integer; InfoType :Integer; var Buffer; BufferSize :Integer) :Integer;
var Rect :TRect absolute Buffer;
    EqCoord :packed record Ra,Dec :Double end absolute Buffer;
    XYCoord :packed record x,y :Integer end absolute Buffer;
    ObjInfoStrings :TZStrings;
begin
   Result := 0;

   case InfoType of
   OI_RECT:
      if wGroup.GetObjectRect(ObjNum, Rect) then Result := SizeOf(TRect);
   OI_RADEC:
      if wGroup.GetObjectRaDec(ObjNum, EqCoord.Ra, EqCoord.Dec) then Result := SizeOf(EqCoord);
   OI_XY:
      if wGroup.GetObjectXY(ObjNum, XYCoord.x, XYCoord.y) then Result := SizeOf(XYCoord);
   OI_DATA:
      Result := wGroup.GetObjectData(ObjNum, Buffer, BufferSize);
   OI_STRINGS: begin
      ObjInfoStrings := TZStrings.Create;
      wGroup.GetObjectStrings(ObjNum, ObjInfoStrings);
      Result := SetStrField(Buffer, BufferSize, ObjInfoStrings.Str);
      if Result<=Length(ObjInfoStrings.Str) then Result := 1 else Result := ObjInfoStrings.Count;
      ObjInfoStrings.Free;
   end;
   OI_NAME:
      Result := SetStrField(Buffer, BufferSize, wGroup.GetObjectName(ObjNum));
   end;
end;

function InitResources(Parent :HWND) :Boolean;
begin
   Result := wGroup.InitResources(Parent);
   if Result then wGroup.ResourcesInited := true;
end;

procedure FreeResources(Parent :HWND);
begin
   wGroup.FreeResources(Parent);
   wGroup.ResourcesInited := false;
end;

procedure RefreshParam(Param :PCommonParameters);
begin
   with Param^ do begin
      wGroup.UT := UT;
      wGroup.ET := ET;
      wGroup.LST := LST;
      wGroup.Latitude := Latitude;
      wGroup.Longitude := Longitude;
      wGroup.TimeZone := TimeZone;
      wGroup.ImCentX := ImCentX;
      wGroup.ImCentY := ImCentY;
      wGroup.ImRadius := ImRadius;
      wGroup.ImYScale := ImYScale;
      wGroup.Zoom := Zoom;
      wGroup.Projection := Projection;
      wGroup.RotAngle := RotAngle;
      wGroup.AzimuthZeroPoint := AzimuthZeroPoint;
      wGroup.MirrorView := MirrorView;
      wGroup.StdColors := StdColors^;
      wGroup.SkyBackgroundChanges := SkyBackgroundChanges;
      wGroup.StdFontHeight := StdFontHeight;
      wGroup.MaxCharWidth := MaxCharWidth;
      wGroup.DrawStars := DrawStars;
      wGroup.MaxStarMag := MaxStarMag;
      wGroup.MaxLabeledStarMag := MaxLabeledStarMag;
      wGroup.MagLevelsCount := MagLevelsCount;
      wGroup.UseStarProperNames := UseStarProperNames;
      wGroup.DrawSolarObj := DrawSolarObj;
      wGroup.DrawSolarObjTypes := DrawSolarObjTypes;
      wGroup.AlwaysHighPrecision := AlwaysHighPrecision;
      wGroup.DrawConBounds := DrawConBounds;
      wGroup.DrawConLines := DrawConLines;
      wGroup.DrawConNames := DrawConNames;
      wGroup.DrawDeepSkyObj := DrawDeepSkyObj;
   end;
   wGroup.OnRefreshParams;
end;

procedure CalculateObjects(ViewMode :TViewMode; ImageRect :TRect;
           MinRa, MaxRa, MinDec, MaxDec :Double; SetObjectNum :TSetObjectNum);
begin
   wGroup.ViewMode := ViewMode;

   wGroup.MinX := ImageRect.Left;
   wGroup.MaxX := ImageRect.Right;
   wGroup.MinY := ImageRect.Top;
   wGroup.MaxY := ImageRect.Bottom;

   wGroup.MinRa := MinRa;
   wGroup.MaxRa := MaxRa;
   wGroup.MinDec := MinDec;
   wGroup.MaxDec := MaxDec;

   wGroup.CalculateObjects(SetObjectNum);
end;

procedure DrawObjects(DC :HDC; const CRect :TRect; GetObjectNum :TGetObjectNum);
begin
   wGroup.DC   := DC;
   wGroup.MinX := CRect.Left;
   wGroup.MaxX := CRect.Right;
   wGroup.MinY := CRect.Top;
   wGroup.MaxY := CRect.Bottom;

   wGroup.DrawObjects(GetObjectNum);
end;

function SelectObjectForSearch(Parent :HWND; var Data :TSearchData) :Integer;
begin
   with wGroup, Data do begin
      Result := SelectObjectForSearch(Parent);
      if Result<>OBJECTNUM_ERROR then begin
         if SearchGroupID='' then begin
            if GetObjectRaDec(Result, Ra, Dec) then
               Move(PChar(ID)^, GroupID, Length(ID)+1)
            else
               Result := OBJECTNUM_ERROR;
         end else begin
            Ra := SearchRa;
            Dec := SearchDec;
            Move(PChar(SearchGroupID)^, GroupID, Length(SearchGroupID)+1);
         end;
      end;
   end;
end;

function EditOptions(Parent :HWND) :Boolean;
begin
   Result := wGroup.EditOptions(Parent);
end;

procedure AdvancedInfo(Parent :HWND; ObjNum, FuncNum :Integer);
begin
   wGroup.AdvancedInfo(Parent, ObjNum, FuncNum);
end;

function AdvancedAction(Parent :HWND) :Boolean;
begin
   Result := wGroup.AdvancedAction(Parent);
end;

end.