unit DateTime;

interface

{ **************************************************************************
  *                                                                        *
  *   .                                         *
  *                                                                        *
  *   StarCalc        *
  * ,         *
  * .                                                                 *
  *                                                                        *
  ************************************************************************** }

const JD_MJD :Double = 2400000.5; {    (JD)  
                                      (MJD) }

{          
  4  1582 .      15  1582 . }
function MJD(Day,Month,Year :Integer; Hour :Double) :Double; stdcall;

{        (MJD) }
procedure CalDat(MJD :Double; var Day,Month,Year :Integer; var Hour :Double); stdcall;

{     MJD }
function GetWeekDay(MJD :Double) :Integer; stdcall;

{    ,    J2000  
   . ( T = (JD-2451545.0)/36525.0 ) }
function GetT(MJD :Double) :Double; stdcall;

{ ,   GetT }
function TToMJD(T :Double) :Double; stdcall;

{     }
function LMST(C_MJD :Double; Lambda :Double) :Double; stdcall;

{ DeltaT = Ephemeris Time - Universal Time
 *
 * The tabulated values of deltaT, in hundredths of a second,
 * were taken from The Astronomical Almanac, page K8.  The program
 * adjusts for a value of secular tidal acceleration ndot. It is -25.8
 * arcsec per century squared for JPL's DE403 ephemeris.
 * ELP2000 and DE200 use the value -23.8946.
 *
 * The tabulated range is 1620.0 through 2004.0.  Bessel's interpolation
 * formula is implemented to obtain fourth order interpolated values at
 * intermediate times.
 * Updated deltaT predictions can be obtained from this network archive:
 *    ftp://maia.usno.navy.mil/ser7/deltat.preds
 *    ftp://maia.usno.navy.mil/ser7/deltat.data
 *
 * For dates earlier than the tabulated range, the program
 * calculates approximate formulae of Stephenson and Morrison
 * or K. M. Borkowski.  These approximations have an estimated
 * error of 15 minutes at 1500 B.C.  They are not adjusted for small
 * improvements in the current estimate of ndot because the formulas
 * were derived from studies of ancient eclipses and other historical
 * information, whose interpretation depends only partly on ndot.
 *
 * A quadratic extrapolation formula, that agrees in value and slope with
 * current data, predicts future values of deltaT.
 *
 * Input Y is the Julian epoch expressed in Julian years.  Y can be
 * found from the Julian date JD by
 *     Y = 2000.0 + (JD - 2451545.0)/365.25
 * See AA page B4.
 *
 * Output double deltat(Y) is ET-UT in seconds.
 *
 *
 * References:
 *
 * Stephenson, F. R., and L. V. Morrison, "Long-term changes
 * in the rotation of the Earth: 700 B.C. to A.D. 1980,"
 * Philosophical Transactions of the Royal Society of London
 * Series A 313, 47-70 (1984)
 *
 * Borkowski, K. M., "ELP2000-85 and the Dynamical Time
 * - Universal Time relation," Astronomy and Astrophysics
 * 205, L8-L10 (1988)
 * Borkowski's formula is derived from eclipses going back to 2137 BC
 * and uses lunar position based on tidal coefficient of -23.9 arcsec/cy^2.
 *
 * Chapront-Touze, Michelle, and Jean Chapront, _Lunar Tables
 * and Programs from 4000 B.C. to A.D. 8000_, Willmann-Bell 1991
 * Their table agrees with the one here, but the entries are
 * rounded to the nearest whole second.
 *
 * Stephenson, F. R., and M. A. Houlden, _Atlas of Historical
 * Eclipse Maps_, Cambridge U. Press (1986)
 }
function GetDeltat(T :Double) :Double; stdcall;

implementation

const SCUtilDLL = 'SCUTILS.DLL';

function MJD(Day,Month,Year :Integer; Hour :Double) :Double; external SCUtilDLL name 'MJD';
procedure CalDat(MJD :Double; var Day,Month,Year :Integer; var Hour :Double); external SCUtilDLL name 'CalDat';
function GetWeekDay(MJD :Double) :Integer; external SCUtilDLL name 'GetWeekDay';
function GetT(MJD :Double) :Double; external SCUtilDLL name 'GetT';
function TToMJD(T :Double) :Double; external SCUtilDLL name 'TToMJD';
function LMST(C_MJD :Double; Lambda :Double) :Double; external SCUtilDLL name 'LMST';
function GetDeltat(T :double) :double; external SCUtilDLL name 'GetDeltat';

end.